@@ -1,6 +1,10 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2018 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2013-2021 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+                        Eberhard Karls Universitaet Tuebingen
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -29,11 +33,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../run/config.h"
 #include "../util/sequence/sequence.h"
 
-using namespace std;
+using std::endl;
+using std::runtime_error;
+using std::string;
+using std::vector;
 
-unique_ptr<Output_format> output_format;
+const uint32_t IntermediateRecord::FINISHED = UINT32_MAX;
 
-void IntermediateRecord::read(BinaryBuffer::Iterator& f)
+void IntermediateRecord::read(BinaryBuffer::Iterator& f, const OutputFormat* output_format)
 {
 	f.read(target_dict_id);
 	/*if (config.global_ranking_targets > 0) {
@@ -42,7 +49,7 @@ void IntermediateRecord::read(BinaryBuffer::Iterator& f)
 		score = s;
 		return;
 	}*/
-	if (*output_format == Output_format::daa)
+	if (*output_format == OutputFormat::daa)
 		f.read(target_oid);
 	f.read(flag);
 	f.read_packed(flag & 3, score);
@@ -68,12 +75,19 @@ void IntermediateRecord::read(BinaryBuffer::Iterator& f)
 	}
 }
 
-interval IntermediateRecord::absolute_query_range() const
+unsigned IntermediateRecord::frame(Loc query_source_len, int align_mode) const {
+	if (align_mode == AlignMode::blastx)
+		return (flag&(1 << 6)) == 0 ? query_begin % 3 : 3 + (query_source_len - 1 - query_begin) % 3;
+	else
+		return 0;
+}
+
+Interval IntermediateRecord::absolute_query_range() const
 {
 	if (query_begin < query_end)
-		return interval(query_begin, query_end + 1);
+		return Interval(query_begin, query_end + 1);
 	else
-		return interval(query_end, query_begin + 1);
+		return Interval(query_end, query_begin + 1);
 }
 
 size_t IntermediateRecord::write_query_intro(TextBuffer& buf, unsigned query_id)
@@ -88,12 +102,12 @@ void IntermediateRecord::finish_query(TextBuffer& buf, size_t seek_pos)
 	*(uint32_t*)(&buf[seek_pos + sizeof(uint32_t)]) = safe_cast<uint32_t>(buf.size() - seek_pos - sizeof(uint32_t) * 2);
 }
 
-void IntermediateRecord::write(TextBuffer& buf, const Hsp& match, unsigned query_id, size_t target_dict_id, size_t target_oid)
+void IntermediateRecord::write(TextBuffer& buf, const Hsp& match, unsigned query_id, DictId target, OId target_oid, const OutputFormat* output_format)
 {
-	const interval oriented_range(match.oriented_range());
-	buf.write((uint32_t)target_dict_id);
-	if (*output_format == Output_format::daa)
-		buf.write((uint32_t)target_oid);
+	const Interval oriented_range(match.oriented_range());
+	buf.write(target);
+	if (*output_format == OutputFormat::daa)
+		buf.write(target_oid);
 	buf.write(get_segment_flag(match));
 	buf.write_packed(match.score);
 	buf.write(match.evalue);
@@ -131,7 +145,7 @@ void IntermediateRecord::finish_file(Consumer& f)
 	f.consume(reinterpret_cast<const char*>(&i), 4);
 }
 
-void Output_format::print_title(TextBuffer &buf, const char *id, bool full_titles, bool all_titles, const char *separator, const EscapeSequences *esc)
+void OutputFormat::print_title(TextBuffer &buf, const char *id, bool full_titles, bool all_titles, const char *separator, const EscapeSequences *esc)
 {
 	if (!all_titles) {
 		if (config.short_seqids)
@@ -167,10 +181,10 @@ void print_hsp(Hsp &hsp, const TranslatedSequence &query)
 	TextBuffer buf;
 	//Pairwise_format().print_match(Hsp_context(hsp, 0, query, "", 0, 0, 0, 0, Sequence()), Search::Config(true), buf);
 	buf << '\0';
-	cout << buf.data() << endl;
+	std::cout << buf.data() << endl;
 }
 
-Output_format* get_output_format()
+OutputFormat* get_output_format()
 {
 	const vector<string> &f = config.output_format;
 	if (f.size() == 0) {
@@ -199,55 +213,61 @@ Output_format* get_output_format()
 		return new Bin1_format;
 	else if (f[0] == "clus")
 		return new Clustering_format(&f[1]);
-	else if (f[0] == "bin")
-		return new Binary_format;
+	else if (f[0] == "edge")
+		return new Output::Format::Edge;
 	else
 		throw std::runtime_error("Invalid output format: " + f[0] + "\nAllowed values: 0,5,xml,6,tab,100,daa,101,sam,102,103,paf");
 }
 
-void init_output()
+OutputFormat* init_output(const int64_t max_target_seqs)
 {
-	output_format = unique_ptr<Output_format>(get_output_format());
+	OutputFormat* output_format = get_output_format();
 	if(config.command == Config::view && (output_format->needs_taxon_id_lists || output_format->needs_taxon_nodes || output_format->needs_taxon_scientific_names))
 		throw runtime_error("Taxonomy features are not supported for the DAA format.");
 
-	if (*output_format == Output_format::daa && config.multiprocessing)
+	if (*output_format == OutputFormat::daa && config.multiprocessing)
 		throw std::runtime_error("The DAA format is not supported in multiprocessing mode.");
-	if (*output_format == Output_format::daa && config.global_ranking_targets)
+	if (*output_format == OutputFormat::daa && config.global_ranking_targets)
 		throw std::runtime_error("The DAA format is not supported in global ranking mode.");
-	if (*output_format == Output_format::taxon && config.toppercent == 100.0 && config.min_bit_score == 0.0)
+	if (*output_format == OutputFormat::taxon && config.toppercent == 100.0 && config.min_bit_score == 0.0)
 		config.toppercent = 10.0;
 	if (config.toppercent == 100.0) {
-		message_stream << "#Target sequences to report alignments for: ";
-		if (config.max_alignments == 0) {
-			config.max_alignments = std::numeric_limits<size_t>::max();
-			message_stream << "unlimited" << endl;
+		if (max_target_seqs >= 0) {
+			message_stream << "#Target sequences to report alignments for: ";
+			if (max_target_seqs == INT64_MAX || max_target_seqs == 0)
+				message_stream << "unlimited";
+			else
+				message_stream << max_target_seqs;
+			message_stream << endl;
 		}
-		else
-			message_stream << config.max_alignments << endl;
 	}
 	else
 		message_stream << "Percentage range of top alignment score to report hits: " << config.toppercent << endl;
 	if (config.frame_shift != 0 && (output_format->hsp_values != HspValues::NONE || config.query_range_culling))
 		output_format->hsp_values = HspValues::TRANSCRIPT;
 	log_stream << "DP fields: " << (unsigned)output_format->hsp_values << endl;
+	return output_format;
 }
 
-void Bin1_format::print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned, const Search::Config& cfg) const {
-	out.write(std::numeric_limits<uint32_t>::max());
-	out.write((uint32_t)query_num);
+void Bin1_format::print_query_intro(Output::Info& info) const {
+	info.out.write(std::numeric_limits<uint32_t>::max());
+	info.out.write((uint32_t)info.query.block_id);
 }
 
-void Bin1_format::print_match(const HspContext& r, const Search::Config &metadata, TextBuffer &out) {
+void Bin1_format::print_match(const HspContext& r, Output::Info& info) {
 	if (r.query_id < r.subject_oid) {
-		out.write((uint32_t)r.subject_oid);
-		out.write(r.bit_score() / std::max(r.query.source().length(), r.subject_len));
+		info.out.write((uint32_t)r.subject_oid);
+		info.out.write(r.bit_score() / std::max(r.query.source().length(), r.subject_len));
 	}
 }
 
-void Binary_format::print_match(const HspContext& r, const Search::Config& metadata, TextBuffer& out)
+namespace Output { namespace Format {
+
+void Edge::print_match(const HspContext& r, Output::Info& info)
 {
-	out.write((uint32_t)metadata.query->block_id2oid(r.query_id));
-	out.write((uint32_t)r.subject_oid);
+	info.out.write(Data{ r.query_oid, r.subject_oid, (float)r.qcovhsp(), (float)r.scovhsp(),
+		config.mmseqs_compat ? (r.evalue() == 0.0 ? r.bit_score() : -r.evalue())
+		: r.corrected_bit_score() });
 }
 
+}}
\ No newline at end of file
