@@ -1,6 +1,10 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2013-2022 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+                        Eberhard Karls Universitaet Tuebingen
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,6 +21,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
 #include <iostream>
+#ifdef WITH_MIMALLOC
+#include <mimalloc-2.0/mimalloc-new-delete.h>
+#endif
 #include "../basic/config.h"
 #include "tools.h"
 #include "../data/reference.h"
@@ -25,10 +32,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../output/recursive_parser.h"
 #include "../util/simd.h"
 #include "../data/dmnd/dmnd.h"
+#include "../util/command_line_parser.h"
 
 using std::cout;
 using std::cerr;
 using std::endl;
+using std::runtime_error;
+using std::string;
 
 void model_seqs();
 void opt();
@@ -48,9 +58,7 @@ void translate();
 void filter_blasttab();
 void show_cbs();
 void reverse();
-void get_medoids_from_tree();
 void roc();
-void merge_tsv();
 void roc_id();
 void makeindex();
 void find_shapes();
@@ -58,23 +66,32 @@ void composition();
 void join();
 void hash_seqs();
 void list_seeds();
-#ifdef WITH_BLASTDB
-void prep_blast_db();
+void prep_db();
+void greedy_vertex_cover();
+#ifdef EXTRA
+void index_fasta();
+void fetch_seq();
+void length_sort();
+void sort();
 #endif
 
 void split();
 namespace Benchmark { DECL_DISPATCH(void, benchmark, ()) }
-namespace Util { namespace Algo { namespace UPGMA { void upgma(); } } }
-namespace Util { namespace Algo { namespace UPGMA_MC { void upgma(); } } }
 namespace Test { int run();
-void simulate_seqs();
-void mutate();
 }
+namespace Cluster {
+void reassign();
+void realign();
+void recluster();
+namespace Incremental {
+}}
 
 int main(int ac, const char* av[])
 {
 	try {
-		config = Config(ac, av);
+		CommandLineParser parser;
+		config = Config(ac, av, true, parser);
+
 
 		switch (config.command) {
 		case Config::help:
@@ -92,10 +109,12 @@ int main(int ac, const char* av[])
 		case Config::view:
 			if (!config.daa_file.empty())
 				view_daa();
+#ifdef EXTRA
 			else if (!config.input_ref_file.empty())
 				view_tsv();
+#endif
 			else
-				throw std::runtime_error("The view command requires either a text (option --in) or DAA (option -a) input file.");
+				throw std::runtime_error("The view command requires a DAA (option -a) input file.");
 			break;
 		case Config::getseq:
 			get_seq();
@@ -103,12 +122,6 @@ int main(int ac, const char* av[])
 		case Config::random_seqs:
 			random_seqs();
 			break;
-		case Config::sort:
-			sort_file();
-			break;
-		case Config::db_stat:
-			db_stat();
-			break;
 		case Config::mask:
 			run_masker();
 			break;
@@ -128,10 +141,11 @@ int main(int ac, const char* av[])
 			pairwise();
 			break;
 		case Config::cluster:
+		case Config::DEEPCLUST:
 			// Why is cluster_similarity not set at the end of the Config constructor?
 			if(!config.cluster_similarity.empty()){
 				string expression = RecursiveParser::clean_expression(&config.cluster_similarity);
-				RecursiveParser rp(nullptr, expression.c_str(), true);
+				RecursiveParser rp(nullptr, expression.c_str());
 				try{
 					rp.evaluate();
 				}
@@ -140,7 +154,7 @@ int main(int ac, const char* av[])
 					throw e;
 				}
 			}
-			Workflow::Cluster::ClusterRegistry::get(config.cluster_algo)->run();
+			Workflow::Cluster::ClusterRegistry::get(config.cluster_algo.get("cascaded"))->run();
 			break;
 		case Config::translate:
 			translate();
@@ -151,39 +165,21 @@ int main(int ac, const char* av[])
 		case Config::show_cbs:
 			show_cbs();
 			break;
-		case Config::simulate_seqs:
-			Test::simulate_seqs();
-			break;
 		case Config::benchmark:
 			Benchmark::benchmark();
 			break;
 		case Config::split:
 			split();
 			break;
-		case Config::upgma:
-			Util::Algo::UPGMA::upgma();
-			break;
-		case Config::upgma_mc:
-			Util::Algo::UPGMA_MC::upgma();
-			break;
 		case Config::regression_test:
 			return Test::run();
 			break;
 		case Config::reverse_seqs:
 			reverse();
 			break;
-		case Config::compute_medoids:
-			get_medoids_from_tree();
-			break;
-		case Config::mutate:
-			Test::mutate();
-			break;
 		case Config::roc:
 			roc();
 			break;
-		case Config::merge_tsv:
-			merge_tsv();
-			break;
 		case Config::rocid:
 			roc_id();
 			break;
@@ -196,11 +192,9 @@ int main(int ac, const char* av[])
 		case Config::HASH_SEQS:
 			hash_seqs();
 			break;
-#ifdef WITH_BLASTDB
-		case Config::prep_blast_db:
-			prep_blast_db();
+		case Config::prep_db:
+			prep_db();
 			break;
-#endif
 		case Config::composition:
 			composition();
 			break;
@@ -210,14 +204,46 @@ int main(int ac, const char* av[])
 		case Config::LIST_SEEDS:
 			list_seeds();
 			break;
+		case Config::CLUSTER_REALIGN:
+			Cluster::realign();
+			break;
+		case Config::GREEDY_VERTEX_COVER:
+			greedy_vertex_cover();
+			break;
+		case Config::CLUSTER_REASSIGN:
+			Cluster::reassign();
+			break;
+		case Config::RECLUSTER:
+			Cluster::recluster();
+			break;
+#ifdef EXTRA
+		case Config::INDEX_FASTA:
+			index_fasta();
+			break;
+        case Config::FETCH_SEQ:
+			fetch_seq();
+			break;
+        case Config::blastn:
+            Search::run();
+            break;
+		case Config::LENGTH_SORT:
+			length_sort();
+			break;
+		case Config::sort:
+			sort();
+			break;
+#endif
 		default:
 			return 1;
 		}
 	}
-	catch(const std::bad_alloc &e) {
+	catch (const std::bad_alloc &e) {
 		cerr << "Failed to allocate sufficient memory. Please refer to the manual for instructions on memory usage." << endl;
 		log_stream << "Error: " << e.what() << endl;
 		return 1;
+	}
+	catch (const FileOpenException&) {
+		return 1;
 	} catch(const std::exception& e) {
         cerr << "Error: " << e.what() << endl;
         log_stream << "Error: " << e.what() << endl;
