@@ -1,6 +1,6 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2020 Max Planck Society for the Advancement of Science e.V.
+Copyright (C) 2020-2022 Max Planck Society for the Advancement of Science e.V.
 
 Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
@@ -35,22 +35,25 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../run/config.h"
 
 using std::vector;
+using std::numeric_limits;
 
 namespace Search {
 namespace DISPATCH_ARCH {
 
 static const int SHORT_QUERY_LEN = 85;
 
-static int ungapped_cutoff(int query_len, const Context& context) {
+static int ungapped_cutoff(int query_len, const WorkSet& context) {
 #ifdef UNGAPPED_SPOUGE
 	return query_len > config.short_query_max_len ? context.cutoff_table(query_len, 50) : context.short_query_ungapped_cutoff;
 #else
-	if (query_len <= config.short_query_max_len)
-		return context.short_query_ungapped_cutoff;
+	if (context.cfg.ungapped_evalue == 0.0)
+		return 0;
+	else if (query_len <= config.short_query_max_len)
+		return context.context.short_query_ungapped_cutoff;
 	else if (query_len <= SHORT_QUERY_LEN && align_mode.query_translated)
-		return context.cutoff_table_short(query_len);
+		return context.cfg.cutoff_table_short(query_len);
 	else
-		return context.cutoff_table(query_len);
+		return context.cfg.cutoff_table(query_len);
 #endif
 }
 
@@ -63,11 +66,11 @@ static int ungapped_window(int query_len) {
 
 static void search_query_offset(const SeedLoc& q,
 	const SeedLoc* s,
-	FlatArray<uint32_t>::ConstIterator hits,
-	FlatArray<uint32_t>::ConstIterator hits_end,
+	FlatArray<uint32_t>::DataConstIterator hits,
+	FlatArray<uint32_t>::DataConstIterator hits_end,
 	WorkSet& work_set)
 {
-	constexpr auto N = vector<Stage1_hit>::const_iterator::difference_type(::DISPATCH_ARCH::SIMD::Vector<int8_t>::CHANNELS);
+	constexpr int N = ::DISPATCH_ARCH::SIMD::Vector<int8_t>::CHANNELS;
 	const SequenceSet& ref_seqs = work_set.cfg.target->seqs(), &query_seqs = work_set.cfg.query->seqs();
 	const Letter* query = query_seqs.data(q);
 
@@ -75,40 +78,43 @@ static void search_query_offset(const SeedLoc& q,
 	int scores[N];
 	std::fill(scores, scores + N, INT_MAX);
 
-	unsigned query_id = UINT_MAX, seed_offset = UINT_MAX;
+	unsigned query_id = UINT_MAX;
+	Loc seed_offset = numeric_limits<Loc>::max();
 #ifdef KEEP_TARGET_ID
 	query_id = q.block_id;
-	seed_offset = (size_t)q.pos - query_seqs.position(query_id, 0);
+	seed_offset = Loc((int64_t)q.pos - query_seqs.position(query_id, 0));
 #else
 	std::pair<size_t, size_t> l = query_seqs.local_position((uint64_t)q); // lazy eval?
 	query_id = (unsigned)l.first;
 	seed_offset = (unsigned)l.second;
 #endif
 	const int query_len = query_seqs.length(query_id);
-	const int score_cutoff = ungapped_cutoff(query_len, work_set.context);
+	const int score_cutoff = ungapped_cutoff(query_len, work_set);
 	const int window = ungapped_window(query_len);
 	const Sequence query_clipped = Util::Seq::clip(query - window, window * 2, window);
 	const int window_left = int(query - query_clipped.data()), window_clipped = (int)query_clipped.length();
 	const unsigned sid = work_set.shape_id;
 	const bool chunked = work_set.cfg.index_chunks > 1;
 	const unsigned hamming_filter_id = work_set.cfg.hamming_filter_id;
-	size_t n = 0, hit_count = 0;
+	int n = 0;
+	size_t hit_count = 0;
 
 	const int interval_mod = config.left_most_interval > 0 ? seed_offset % config.left_most_interval : window_left, interval_overhang = std::max(window_left - interval_mod, 0);
 
-	for (FlatArray<uint32_t>::ConstIterator i = hits; i < hits_end; i += n) {
+	for (FlatArray<uint32_t>::DataConstIterator i = hits; i < hits_end; i += n) {
 
-		n = std::min(N, hits_end - i);
-		for (size_t j = 0; j < n; ++j)
+		n = std::min(N, int(hits_end - i));
+		for (int j = 0; j < n; ++j)
 			subjects[j] = ref_seqs.data(s[*(i + j)]) - window_left;
-		DP::window_ungapped_best(query_clipped.data(), subjects, n, window_clipped, scores);
+		if (score_cutoff)
+			DP::window_ungapped_best(query_clipped.data(), subjects, n, window_clipped, scores);
 
 		if (config.global_ranking_targets)
-			for (size_t j = 0; j < n; ++j)
+			for (int j = 0; j < n; ++j)
 				if (scores[j] == UCHAR_MAX)
 					scores[j] = ::ungapped_window(query_clipped.data(), subjects[j], window_clipped);
 
-		for (size_t j = 0; j < n; ++j) {
+		for (int j = 0; j < n; ++j) {
 			if (scores[j] > score_cutoff) {
 #ifdef UNGAPPED_SPOUGE
 				std::pair<size_t, size_t> l = ref_seqs::data_->local_position(s[*(i + j)]);
@@ -116,13 +122,15 @@ static void search_query_offset(const SeedLoc& q,
 					continue;
 #endif
 				work_set.stats.inc(Statistics::TENTATIVE_MATCHES2);
-				if (left_most_filter(query_clipped + interval_overhang, subjects[j] + interval_overhang, window_left - interval_overhang, shapes[sid].length_, work_set.context, sid == 0, sid, score_cutoff, chunked, hamming_filter_id)) {
+				if (work_set.cfg.minimizer_window || config.sketch_size
+					|| left_most_filter(query_clipped + interval_overhang, subjects[j] + interval_overhang, window_left - interval_overhang, shapes[sid].length_, work_set.context, sid == 0, sid, score_cutoff, chunked, hamming_filter_id)) {
 					work_set.stats.inc(Statistics::TENTATIVE_MATCHES3);
 					if (hit_count++ == 0)
 						*work_set.out = SerializerTraits<Hit>::make_sentry(query_id, seed_offset);
 #ifdef KEEP_TARGET_ID
 					if(config.global_ranking_targets)
-						*work_set.out = { query_id, (uint64_t)s[*(i + j)].block_id, seed_offset, (uint16_t)scores[j] };
+						//*work_set.out = { query_id, (uint64_t)s[*(i + j)].block_id, seed_offset, (uint16_t)scores[j] };
+						*work_set.out = { query_id, (uint64_t)s[*(i + j)].pos, seed_offset, (uint16_t)scores[j], s[*(i + j)].block_id };
 					else
 #ifdef HIT_KEEP_TARGET_ID
 						*work_set.out = { query_id, (uint64_t)s[*(i + j)].pos, seed_offset, (uint16_t)scores[j], s[*(i + j)].block_id };
@@ -140,8 +148,8 @@ static void search_query_offset(const SeedLoc& q,
 
 static void FLATTEN search_tile(
 	const FlatArray<uint32_t> &hits,
-	uint32_t query_begin,
-	uint32_t subject_begin,
+	int32_t query_begin,
+	int32_t subject_begin,
 	const SeedLoc* q,
 	const SeedLoc* s,
 	WorkSet& work_set)
@@ -150,7 +158,7 @@ static void FLATTEN search_tile(
 	const uint32_t query_count = (uint32_t)hits.size();
 	const SeedLoc* q_begin = q + query_begin, *s_begin = s + subject_begin;
 	for (uint32_t i = 0; i < query_count; ++i) {
-		FlatArray<uint32_t>::ConstIterator r1 = hits.begin(i), r2 = hits.end(i);
+		FlatArray<uint32_t>::DataConstIterator r1 = hits.begin(i), r2 = hits.end(i);
 		if (r2 == r1)
 			continue;
 		search_query_offset(q_begin[i], s_begin, r1, r2, work_set);
@@ -169,35 +177,114 @@ static void all_vs_all(const FingerPrint* a, uint32_t na, const FingerPrint* b,
 	}
 }
 
+static void all_vs_all_self(const FingerPrint* a, uint32_t na, FlatArray<uint32_t>& out, unsigned hamming_filter_id) {
+	for (uint32_t i = 0; i < na; ++i) {
+		const FingerPrint e = a[i];
+		out.next();
+		for (uint32_t j = i + 1; j < na; ++j)
+			if (e.match(a[j]) >= hamming_filter_id)
+				out.push_back(j);
+	}
+}
+
 static void load_fps(const SeedLoc* p, size_t n, Container& v, const SequenceSet& seqs)
 {
 	v.clear();
 	v.reserve(n);
 	const SeedLoc* end = p + n;
-	for (; p < end; ++p)
+	for (; p < end; ++p) {
 		v.emplace_back(seqs.data(*p));
+	}
 }
 
-void FLATTEN stage1(const SeedLoc* q, size_t nq, const SeedLoc* s, size_t ns, WorkSet& work_set)
+void FLATTEN stage1(const SeedLoc* q, int32_t nq, const SeedLoc* s, int32_t ns, WorkSet& work_set)
 {
 #ifdef __APPLE__
 	thread_local Container vq, vs;
 #else
 	Container& vq = work_set.vq, &vs = work_set.vs;
 #endif
+	
+	const int32_t tile_size = config.tile_size;
+	load_fps(s, ns, vs, work_set.cfg.target->seqs());
 	work_set.stats.inc(Statistics::SEED_HITS, nq * ns);
 	load_fps(q, nq, vq, work_set.cfg.query->seqs());
-	load_fps(s, ns, vs, work_set.cfg.target->seqs());
-
-	const size_t tile_size = config.tile_size;
-	for (size_t i = 0; i < vq.size(); i += tile_size) {
-		for (size_t j = 0; j < vs.size(); j += tile_size) {
+	const int32_t qs = (int32_t)vq.size(), ss = (int32_t)vs.size();
+	for (int32_t i = 0; i < qs; i += tile_size) {
+		for (int32_t j = 0; j < ss; j += tile_size) {
 			work_set.hits.clear();
-			all_vs_all(vq.data() + i, (uint32_t)std::min(tile_size, vq.size() - i), vs.data() + j, (uint32_t)std::min(tile_size, vs.size() - j), work_set.hits, work_set.cfg.hamming_filter_id);
+			all_vs_all(vq.data() + i, std::min(tile_size, qs - i), vs.data() + j, std::min(tile_size, ss - j), work_set.hits, work_set.cfg.hamming_filter_id);
 			search_tile(work_set.hits, i, j, q, s, work_set);
 		}
 	}
+}
 
+void FLATTEN stage1_lin(const SeedLoc* q, int32_t nq, const SeedLoc* s, int32_t ns, WorkSet& work_set)
+{
+#ifdef __APPLE__
+	thread_local Container vq, vs;
+#else
+	Container& vq = work_set.vq, &vs = work_set.vs;
+#endif
+
+	const int32_t tile_size = config.tile_size;
+	load_fps(q, 1, vq, work_set.cfg.query->seqs());
+	load_fps(s, ns, vs, work_set.cfg.target->seqs());
+	work_set.stats.inc(Statistics::SEED_HITS, ns);
+	
+	const int32_t ss = (int32_t)vs.size();
+	for (int32_t j = 0; j < ss; j += tile_size) {
+		work_set.hits.clear();
+		all_vs_all(vq.data(), 1, vs.data() + j, std::min(tile_size, ss - j), work_set.hits, work_set.cfg.hamming_filter_id);
+		search_tile(work_set.hits, 0, j, q, s, work_set);
+	}
+}
+
+void FLATTEN stage1_lin_ranked(const SeedLoc* q, int32_t nq, const SeedLoc* s, int32_t ns, WorkSet& work_set)
+{
+#ifdef __APPLE__
+	thread_local Container vq, vs;
+#else
+	Container& vq = work_set.vq, &vs = work_set.vs;
+#endif
+
+	const int32_t tile_size = config.tile_size;
+	const int32_t ranking = work_set.kmer_ranking->highest_ranking(q, q + nq);
+	load_fps(q + ranking, 1, vq, work_set.cfg.query->seqs());
+	load_fps(s, ns, vs, work_set.cfg.target->seqs());
+	work_set.stats.inc(Statistics::SEED_HITS, ns);
+
+	const int32_t ss = (int32_t)vs.size();
+	for (int32_t j = 0; j < ss; j += tile_size) {
+		work_set.hits.clear();
+		all_vs_all(vq.data(), 1, vs.data() + j, std::min(tile_size, ss - j), work_set.hits, work_set.cfg.hamming_filter_id);
+		search_tile(work_set.hits, ranking, j, q, s, work_set);
+	}
+}
+
+void FLATTEN stage1_self(const SeedLoc* q, int32_t nq, const SeedLoc* s, int32_t ns, WorkSet& work_set)
+{
+#ifdef __APPLE__
+	thread_local Container vs;
+#else
+	Container& vs = work_set.vs;
+#endif
+
+	const int32_t tile_size = config.tile_size;
+	load_fps(s, ns, vs, work_set.cfg.target->seqs());
+
+	work_set.stats.inc(Statistics::SEED_HITS, ns*(ns - 1) / 2);
+	const int32_t ss = (int32_t)vs.size();
+	for (int32_t i = 0; i < ss; i += tile_size) {
+		work_set.hits.clear();
+		all_vs_all_self(vs.data() + i, std::min(tile_size, ss - i), work_set.hits, work_set.cfg.hamming_filter_id);
+		search_tile(work_set.hits, i, i, s, s, work_set);
+		for (int32_t j = i + tile_size; j < ss; j += tile_size) {
+			work_set.hits.clear();
+			all_vs_all(vs.data() + i, std::min(tile_size, ss - i), vs.data() + j, std::min(tile_size, ss - j), work_set.hits, work_set.cfg.hamming_filter_id);
+			search_tile(work_set.hits, i, j, s, s, work_set);
+		}
+	}
 }
 
 }}
\ No newline at end of file
