@@ -26,28 +26,23 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 using namespace std;
 
-Command_line_parser & Command_line_parser::add(const Options_group & group)
-{
-	for (PtrVector<Option_base>::const_iterator i = group.options.begin(); i != group.options.end(); ++i) {
-		map_[(*i)->id] = *i;
-		map_short_[(*i)->short_id] = *i;
-	}
-	groups_.push_back(&group);
-	return *this;
+OptionsGroup& CommandLineParser::add_group(const char *title, const std::vector<unsigned>& commands, bool disabled) {
+	groups_.emplace_back(title, commands, disabled, this);
+	return groups_.back();
 }
 
-Command_line_parser & Command_line_parser::add_command(const char * s, const char *desc, unsigned code)
+CommandLineParser & CommandLineParser::add_command(const char * s, const char *desc, unsigned code)
 {
 	command_codes_[s] = code;
 	commands_.emplace_back(s, desc);
 	return *this;
 }
 
-void Command_line_parser::store_option(const vector<string> &v)
+void CommandLineParser::store_option(const vector<string> &v, unsigned command)
 {
 	if (v.size() == 0)
 		return;
-	Option_base* o = nullptr;
+	OptionBase* o = nullptr;
 	string id;
 	vector<string> v2;
 
@@ -55,12 +50,12 @@ void Command_line_parser::store_option(const vector<string> &v)
 		throw runtime_error("Invalid option syntax.");
 	else if (v[0].substr(0, 2) == "--") {
 		id = v[0].substr(2);
-		map<string, Option_base*>::const_iterator i = map_.find(id);
+		map<string, OptionBase*>::const_iterator i = map_.find(id);
 		if (i != map_.end())
 			o = i->second;
 	} else if (v[0][0] == '-') {
 		id = v[0][1];
-		map<char, Option_base*>::const_iterator i = map_short_.find(v[0][1]);
+		map<char, OptionBase*>::const_iterator i = map_short_.find(v[0][1]);
 		if (i != map_short_.end())
 			o = i->second;
 		if (v[0].length() > 2)
@@ -68,15 +63,19 @@ void Command_line_parser::store_option(const vector<string> &v)
 	} else
 		throw runtime_error("Command line options must begin with - or --.");
 	
-	if(o == nullptr || o->disabled)
+	if (o == nullptr || o->disabled)
 		throw runtime_error("Invalid option: " + id);
+#ifndef EXTRA
+	else if (find(o->group->commands.begin(), o->group->commands.end(), command) == o->group->commands.end())
+		throw runtime_error("Option is not permitted for this workflow: " + id);
+#endif
 	else {
 		v2.insert(v2.end(), v.begin() + 1, v.end());
 		o->read(v2);
 	}
 }
 
-void Command_line_parser::store(int count, const char ** str, unsigned &command)
+void CommandLineParser::store(int count, const char ** str, unsigned &command)
 {
 	if (count < 2)
 		throw runtime_error("Syntax: diamond COMMAND [OPTIONS]. To print help message: diamond help");
@@ -88,21 +87,21 @@ void Command_line_parser::store(int count, const char ** str, unsigned &command)
 		throw runtime_error("Invalid command: " + cmd + ". To print help message: diamond help");
 	command = it->second;
 
-	for (map<string, Option_base*>::const_iterator i = map_.begin(); i != map_.end(); ++i)
+	for (map<string, OptionBase*>::const_iterator i = map_.begin(); i != map_.end(); ++i)
 		i->second->set_default();
 
 	vector<string> v;
 	for (int i = 2; i < count; ++i) {
 		if (str[i][0] == '-' && strlen(str[i]) > 1 && !isdigit(str[i][1])) {
-			store_option(v);
+			store_option(v, command);
 			v.clear();
 		}
 		v.push_back(str[i]);
 	}
-	store_option(v);
+	store_option(v, command);
 }
 
-void Command_line_parser::print_help()
+void CommandLineParser::print_help()
 {
 	static const size_t col1_width = 25;
 	cout << "Syntax: diamond COMMAND [OPTIONS]" << endl << endl;
@@ -111,20 +110,28 @@ void Command_line_parser::print_help()
 		if (i->second != "")
 			cout << i->first << '\t' << i->second << endl;
 	cout << endl;
-	for (vector<const Options_group*>::const_iterator i = groups_.begin(); i != groups_.end(); ++i) {
-		if ((*i)->title == "")
+	for (const auto& g : groups_) {
+		if (g.title == "")
 			continue;
-		cout << (*i)->title << ":" << endl;
-		for (vector<Option_base*>::const_iterator j = (*i)->options.begin(); j != (*i)->options.end(); ++j) {
-			if((*j)->desc.empty())
+		cout << g.title << ":" << endl;
+		for (const auto& option : g.options) {
+			if(option->desc.empty())
 				continue;
-			string col1 = "--" + (*j)->id;
-			if ((*j)->short_id)
-				col1 += string(" (-") + (*j)->short_id + ")";
+			string col1 = "--" + option->id;
+			if (option->short_id)
+				col1 += string(" (-") + option->short_id + ")";
 			col1.append(max(col1_width, col1.length()) - col1.length(), ' ');
-			cout << col1 << (*j)->desc << endl;
+			cout << col1 << option->desc << endl;
 		}
 		cout << endl;
 	}
 	cout << "Online documentation at http://www.diamondsearch.org" << endl;
 }
+
+void CommandLineParser::require(const char* option) {
+	auto it = map_.find(option);
+	if (it == map_.end())
+		throw std::runtime_error("Unknown option.");
+	if(!it->second->present())
+		throw std::runtime_error("Missing parameter: --" + it->second->id + "/-" + it->second->short_id);
+}
\ No newline at end of file
